/**
 * \addtogroup Timers
 * @{
 */

/*****************************************************************************/
/**
 * \file Timers.c
 * Quad, millisecond-precision pseudo-timer routines for 8051 and Keil C51
 * compiler.
 * \author Copyright (c) 2010, Murray R. Van Luyn. <vanluynm@iinet.net.au>
 * \version 0.3
 * \date 19-06-10
 */
/*****************************************************************************/

/*****************************************************************************
 *
 *     THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 *     OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *     WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *     ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 *     DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *     DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 *     GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *     INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *     WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *     NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/

#include "Timers.h"


/*****************************************************************************
 *
 *                            Static Global Variables
 *
 *****************************************************************************/
static volatile unsigned int timer_0_count = 0;
static volatile unsigned int timer_1_count = 0;
static volatile unsigned int timer_2_count = 0;
static volatile unsigned int timer_3_count = 0;


/*****************************************************************************
 *
 *                         Private Function Prototypes
 *
 *****************************************************************************/
//void timers_timer_0_isr(void) interrupt 1 using 1;


/*****************************************************************************
 *
 *                        Public Function Implementation
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *                                timers_init()              
 *
 *****************************************************************************/
void timers_init(void)
{

#if (TIMERS_X2_MODE == 1)
    /* 
     * Set T0X2 bit 1 of CKCON so Timer 0 clock stays in std. mode with
     * 12 clock periods per peripheral clock cycle.
     */
    CKCON |= 0x02;
#endif

    /* Stop timer 0. */
    TR0  = 0;          

    /* Set timer/ counter 0 as mode 1, 16 bit timer. */
    TMOD &= 0xF0;
    TMOD |= 0x01;

    /* Preload timer 0 for 1ms periodic interrupt. */
    TH0 = (unsigned char)(TIMER_0_RELOAD >> 8);
    TL0 = (unsigned char)(TIMER_0_RELOAD & 0xFF);

    /* Start timer 0 running again. */
    TR0 = 1;

    /* Enable timer 0 overflow interrupt. */
    ET0 = 1;

    /* Enable global interrupt. */
    EA = 1;

}

/*****************************************************************************
 *
 *                           timers_restart_timer()              
 *
 *****************************************************************************/
void timers_restart_timer(unsigned char timer, unsigned int period)
{
    /* Disable timer 0 overflow interrupt. */
    ET0 = 0;

    switch (timer)
    {
        case 0:
        {
            /* Set millisecond counter for pseudo-timer 0. */
            timer_0_count = period;

            break;
        }

        case 1:
        {
            /* Set millisecond counter for pseudo-timer 1. */
            timer_1_count = period;

            break;
        }

        case 2:
        {
            /* Set millisecond counter for pseudo-timer 2. */
            timer_2_count = period;

            break;
        }

        case 3:
        {
            /* Set millisecond counter for pseudo-timer 3. */
            timer_3_count = period;

            break;
        }

        default:

            break;

    }

    /* Re-enable timer 0 overflow interrupt. */
    ET0 = 1;

    return;
}

/*****************************************************************************
 *
 *                           timers_is_expired_timer()              
 *
 *****************************************************************************/
bit timers_is_expired_timer(unsigned char timer)
{
    /* Disable timer 0 overflow interrupt. */
    ET0 = 0;

    switch (timer)
    {
        case 0:
        {
            /* Check whether pseudo-timer 0 has expired. */
            if (timer_0_count == 0)
            {
                /* Re-enable timer 0 overflow interrupt. */
                ET0 = 1;

                return 1;
            }
            break;
        }

        case 1:
        {
            /* Check whether pseudo-timer 1 has expired. */
            if (timer_1_count == 0)
            {
                /* Re-enable timer 0 overflow interrupt. */
                ET0 = 1;

                return 1;
            }
            break;
        }

        case 2:
        {
            /* Check whether pseudo-timer 2 has expired. */
            if (timer_2_count == 0)
            {
                /* Re-enable timer 0 overflow interrupt. */         
                ET0 = 1;

                return 1;
            }
            break;
        }

        case 3:
        {
            /* Check whether pseudo-timer 3 has expired. */
            if (timer_3_count == 0)
            {
                /* Re-enable timer 0 overflow interrupt. */           
                ET0 = 1;

                return 1;
            }
            break;
        }

        default:
        {
            /* Re-enable timer 0 overflow interrupt. */         
            ET0 = 1;

            return 0;

            break;
        }
    }

    /* Re-enable timer 0 overflow interrupt. */          
    ET0 = 1;

    /* Return false if specified pseudo-timer has not expired. */
    return 0;
}


/*****************************************************************************
 *
 *                        Private Function Implementation
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *                             timers_timer_0_isr()  
 *
 *****************************************************************************/
void timers_timer_0_isr(void) interrupt 1 using 1
{
    /* Reload timer 0 for 1ms periodic interrupt. */ 
    TH0 = (unsigned char)(TIMER_0_RELOAD >> 8);
    TL0 = (unsigned char)(TIMER_0_RELOAD & 0xFF);

    /* Decrement all non-zero timer millisecond counters. */
    if (timer_0_count) timer_0_count--;
    if (timer_1_count) timer_1_count--;
    if (timer_2_count) timer_2_count--;
    if (timer_3_count) timer_3_count--;

    return;
}


/** @} */
