/**
 * \addtogroup Timers
 * @{
 */

/*****************************************************************************/
/**
 * \file Timers.h
 * Quad, millisecond-precision pseudo-timer routines for 8051 and Keil C51
 * compiler.
 * \author Copyright (c) 2010, Murray R. Van Luyn. <vanluynm@iinet.net.au>
 * \version 0.3
 * \date 19-06-10
 */
/*****************************************************************************/

/*****************************************************************************
 *
 *     THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 *     OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *     WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *     ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 *     DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *     DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 *     GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *     INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *     WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *     NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/

#ifndef TIMERS_H
#define TIMERS_H

#include "REG51.h"


/*****************************************************************************
 *
 *                           Constant Definitions
 *
 *****************************************************************************/

/** 
 * Hardware crystal frequency in Hertz.
 * Adjust \b TIMERS_XTAL to match target hardware crystal frequency in Hertz.
 * \hideinitializer
 */
#define TIMERS_XTAL          18432000    

/** 
 * X2 mode support binary switch.
 * Set \b TIMERS_X2_MODE when CPU X2 clock mode has been elsewhere
 * engaged.
 * \hideinitializer
 */
#define TIMERS_X2_MODE       0

#if (TIMERS_X2_MODE == 1)
  /** 
   * Clock control special function register name redefinition.
   * Uncomment this redefinition for the original CKCON clock
   * control register in circumstances where your device-specific
   * SFR register declaration header file inconveniently renames
   * it as CKCON0.
   * \hideinitializer
   */
//  #define CKCON              CKCON0
#endif

/*
 * Timer 0 mode 1 clock rate.
 */
#define TIMERS_T1_CLOCK      (TIMERS_XTAL / 12uL) 

/*
 * Timer 0 reload calculation defines.
 */
#define CYCLES_FOR_OVERFLOW 65536u
#define CYCLES_FOR_1MS    (0.001  * TIMERS_T1_CLOCK)
#define TIMER_0_RELOAD    (unsigned int)(CYCLES_FOR_OVERFLOW - CYCLES_FOR_1MS)

#ifdef REG51_H
  #if (TIMERS_X2_MODE == 1)
    /*
     * Flag as an error, any attempt to use the X2 operating mode,
     * whilst the module still includes it's original 8051 register
     * declaration header file REG51.h. 
     */
    #error X2 operating mode requires prior substitution of generic 8051   
    #error register declaration header file Reg51.h, with a more machine   
    #error specific register declaration header file, from a device that  
    #error supports the X2 CPU clock operating mode. 
  #endif
#endif


/*****************************************************************************
 *
 *                           Global Variable Declarations
 *
 *****************************************************************************/


/*****************************************************************************
 *
 *                           Public Function Prototypes
 *
 *****************************************************************************/
/**
 * Quad, millisecond-precision pseudo-timer routine initialisation function.
 * Initialises the 8051 timer 0 for 1ms periodic interrupt.
 * Must be called once to set-up hardware during program initialisation.
 * \param -
 * \return -
 */
extern void timers_init(void);

/**
 * Quad, millisecond-precision pseudo-timer routine restart function.
 * Restarts selected pseudo-timer for expiry after given countdown period.
 * \param timer An \b unsigned \b char indicating which timer to restart.
 * \param period An \b unsigned \b int specifying period for timer to run.
 * \return -
 */
extern void timers_restart_timer(unsigned char timer, unsigned int period);

/**
 * Quad, millisecond-precision pseudo-timer routine expired test function.
 * Returns true when selected pseudo-timer has expired.
 * \param timer An \b unsigned \b char indicating which timer to test.
 * \return A \b bit returning true when timer expired.
 */
extern bit timers_is_expired_timer(unsigned char timer);


#endif

/** @} */
